## Native Backend

- `@always_inline`: Always inline the annotated function.

- `@cpp_export`:

  - On a class: The class is accessed by C++ and so the fields cannot be
    reordered and must follow C++ layout rules (fields must be naturally aligned
    - i.e. a 32-bit value must be aligned on a 32-bit boundary and bools are
    8-bit aligned).  Note that this is still a Skip class which must be
    allocated by Skip to get a proper vtable.

    Note that currently @cpp_export classes must be final and cannot extend base
    classes.

  - On a function: Export the function for use by C++. A name can be provided as
    a parameter.

- `@cpp_extern`: The annotated function is implemented in an external language
  (like C++) to be linked in later. A name can be provided as a parameter.

- `@cpp_runtime`: This native function is declared in the runtime and should
   not be declared by the compiler. A name can be provided as a parameter.

   The only difference between @cpp_runtime and @cpp_extern, is that
   the NBE creates an llvm decl for each @cpp_extern, to which call sites
   get bound. Functions with @cpp_runtime annotations are expected to have
   matching decls in preamble.ll, generated by gen_preamble.

- `@cpp_virtual`: (Internal only): Do not devirtualize calls to this method.

- `@debug`: Turn off CSE optimization for this function.

- `@disasm`: Mark a function as a candidate during disassembly.

- `@gc`: Perform a garbage-collect when the annotated function returns.
  Implies @no_inline.

- `@no_gc`: Prevent any heuristics from automatically garbage collecting
  when this function returns.

- `@hhvm_array`: Similar to `@hhvm_import` mark a class as an imported proxy
  class with copy-on-write semantics.  Internal API functions use the name
  `Array`.

- `@hhvm_extern`: Similar to `@cpp_extern` the annotated function is
  implemented in an external language (like C++) to be linked in later.  The
  default name is mangled based on its parameters like a C++ template function.

- `@hhvm_export`:
  - On a function: Export a function so it is visible from HHVM. Implies
  @no_inline and @gc. A name can be provided as a parameter.

  - On a class: Currently not supported.

  - On a method: Currently not supported.

- `@hhvm_import`:

  - On a function: Import an HHVM function. Implies @no_inline. A name can be
    provided as a parameter.

  - On a class: Mark a class as an imported proxy class.  Fields on the class
    are turned into getters and setters.  Internal API functions use the name
    `Object`.

  - On a method: Import an HHVM method on an @hhvm_import class. A name can be
    provided as a parameter.

= `@hhvm_shape`: On a class behaves like `@hhvm_import` with the following exceptions:

  - Getters are created but setters are not.

  - Uses the internal API for Arrays

- `@intrinsic`: Indicates a native function which will be implemented by the
  compiler itself rather than the runtime.

- `@may_alloc`: on a @cpp_runtime or @cpp_extern function, informs the NBE
  that the function may allocate on the Obstack, which optimizes placement
  of gc safepoints. Ignored on normal skip functions. If not present, NBE
  trusts (but does not check) that the call does not allocate on the Obstack.

- `@no_inline`: Prevent any heuristics from automatically inlining this
  function.

- `@no_return`: Mark this function as one that never returns.  Only allowed
  on native prototypes.

- `@no_throw`: Mark this function as one that never throws.

## Documentation

- `@synonym("name")`

- `@deprecated("reason")`

- `@example("name")`
